from .__init__ import ctypes, \
    SDL_FUNC, SDL_SET_CURRENT_DLL, SDL_DLL

SDL_SET_CURRENT_DLL(SDL_DLL)

SDL_FALSE = 0
SDL_TRUE = 1

SDL_bool = ctypes.c_bool

Sint8 = ctypes.c_int8
SDL_MAX_SINT8 = 0x7F
SDL_MIN_SINT8 = ~0X7F

Uint8 = ctypes.c_uint8
SDL_MAX_UINT8 = 0xFF
SDL_MIN_UINT8 = 0X00

Sint16 = ctypes.c_int16
SDL_MAX_SINT16 = 0x7FFF
SDL_MIN_SINT16 = ~0X7FFF

Uint16 = ctypes.c_uint16
SDL_MAX_UINT16 = 0xFFFF
SDL_MIN_UINT16 = 0X0000

Sint32 = ctypes.c_int32
SDL_MAX_SINT32 = 0x7FFFFFFF
SDL_MIN_SINT32 = ~0X7FFFFFFF

Uint32 = ctypes.c_uint32
SDL_MAX_UINT32 = 0xFFFFFFFF
SDL_MIN_UINT32 = 0X00000000

Sint64 = ctypes.c_int64
SDL_MAX_SINT64 = 0x7FFFFFFFFFFFFFFF
SDL_MIN_SINT64 = ~0X7FFFFFFFFFFFFFFF

Uint64 = ctypes.c_uint64
SDL_MAX_UINT64 = 0xFFFFFFFFFFFFFFFF
SDL_MIN_UINT64 = 0X0000000000000000

SDL_Time = Sint64
SDL_MAX_TIME = SDL_MAX_SINT64
SDL_MIN_TIME = SDL_MIN_SINT64

SDL_FLT_EPSILON = 1.1920928955078125e-07

SDL_FunctionPointer = ctypes.CFUNCTYPE(None)